package org.fuseri.modulelanguageschool;

import lombok.RequiredArgsConstructor;
import org.fuseri.modulelanguageschool.course.Course;
import org.fuseri.modulelanguageschool.course.CourseService;
import org.fuseri.modulelanguageschool.course.Language;
import org.fuseri.modulelanguageschool.course.ProficiencyLevel;
import org.fuseri.modulelanguageschool.lecture.Lecture;
import org.fuseri.modulelanguageschool.lecture.LectureService;
import org.fuseri.modulelanguageschool.user.Address;
import org.fuseri.modulelanguageschool.user.User;
import org.fuseri.modulelanguageschool.user.UserService;
import org.fuseri.modulelanguageschool.user.UserType;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

@RequiredArgsConstructor
@Component
public class DataInitializer implements ApplicationRunner {

    private final UserService userService;
    private final CourseService courseService;
    private final LectureService lectureService;

    @Override
    public void run(ApplicationArguments args) {
        User user = User.builder()
                .email("test@email.com")
                .firstName("John")
                .lastName("Doe")
                .username("johnD")
                .password("password")
                .userType(UserType.ADMIN).address(Address.builder()
                        .city("Brno")
                        .country("CZ")
                        .street("Hrnčírska")
                        .houseNumber("99")
                        .build())
                .build();

        userService.create(user);

        Course course = Course.builder()
                .name("English 101")
                .capacity(10)
                .proficiencyLevel(ProficiencyLevel.A2)
                .language(Language.ENGLISH)
                .build();

        courseService.create(course);

        Lecture lecture = Lecture.builder()
                .course(course).user(List.of(user)).
                from(LocalDateTime.of(2023, 06, 24, 16, 0, 0))
                .to(LocalDateTime.of(2023, 06, 24, 18, 0, 0))
                .build();

        lectureService.create(lecture);
    }
}
